require("tools")

local interval = tonumber(c8y.config:value("interval", 0))
local templates = {}
local timer
function init()
  print("init")
  print("interval:", interval)
  templates.sendAll = c8y.templates:addRequest('POST,/measurement/measurements,application/vnd.com.nsn.cumulocity.measurement+json,,%%,NUMBER NUMBER NUMBER NOW STRING,"{""c8y_SignalStrength"":{""rssi"":{""value"":%%,""unit"":""dBm""},""ECN0"":{""value"": %%},""RCSP"":{""value"":%%}},""time"":""%%"",""source"":{ ""id"":""%%""},""type"":""c8y_SignalMeasurement""}"')
  templates.sendWithoutEcn0 = c8y.templates:addRequest('POST,/measurement/measurements,application/vnd.com.nsn.cumulocity.measurement+json,,%%,NUMBER NUMBER NOW STRING,"{""c8y_SignalStrength"":{""rssi"":{""value"":%%,""unit"":""dBm""},""RCSP"":{""value"":%%}},""time"":""%%"",""source"":{ ""id"":""%%""},""type"":""c8y_SignalMeasurement""}"')
end

function postInit()
  print("postInit")
end

function getRdb(parameter)
  local file = io.popen("rdb get " .. parameter)
  local value = file:read("*n")
  file:close()
  return value
end

function readRssi()
  local rssi = getRdb("wwan.0.radio.information.signal_strength")
  return rssi
end

function readEcn0()
  local valid = getRdb("wwan.0.system_network_status.ECN0_valid")
  if valid == 1 then
    local ecn0 = getRdb("wwan.0.system_network_status.ECN0s0")
    return ecn0
  end
  return nil
end

function readRcsp()
  local rcsp = getRdb("wwan.0.system_network_status.RSCPs0")
  return rcsp
end

function sendMeasurements()
  local rssi = readRssi()
  local ecn0 = readEcn0()
  local rcsp = readRcsp()
  if rssi and rcsp then
    local response
    if ecn0 then
      response = templates.sendAll:send({rssi, ecn0, rcsp, c8y.device})
    else
      response = templates.sendWithoutEcn0:send({rssi, rcsp, c8y.device})
    end
    return response.httpCode == 200
  end
end

function safelySendMeasurements()
 local timer = c8y.timer.start(5, function(timer)
    local result = sendMeasurements()
    if result then
    	timer:stop()
    end
  end)
end

function start()
	-- update interval value every 5 sec
	c8y.timer.start(5, function()
		local n = tonumber(c8y.config:value("interval", 0))
		if n ~= interval then
			print('interval changed '..interval..' -> '..n)
			interval = n
		end
	end)
	safelySendMeasurements()
end

local previousTime = os.time()
function run()
  if interval > 0 then
    local currentTime = os.time()
    if os.difftime(currentTime, previousTime) >= interval then
      --  print("rssi:" .. readRssi(), "ecn0:" .. readEcn0(), "rcsp:" .. readRcsp())
      sendMeasurements()
      previousTime = currentTime
    end
  end
end

